#!/bin/sh
#
# Prepare archive with test fixtures for tree-sitter unit-tests
#
# The upstream CI is using `master` to fetch the grammars; see `script/fetch-fixtures`.
# Replace with the exact commits or tags to make reproducible.
# Make sure to check the licenses for all the projects we download here.
set -e

DESTDIR="$(dirname "$0" | xargs readlink -f)"
VERSION=$(rpmspec --query --qf '%{version}' --srpm "$DESTDIR/rust-tree-sitter-cli.spec")
MODE=fetch

while [ "$#" -gt 0 ]; do
    case $1 in
        [0-9]*)
            VERSION="$1";;
        --update)
            MODE=update;;
        *)
            break;;
    esac
    shift
done

if [ -z "$VERSION" ]; then
    echo "Cannot determine the tree-sitter-cli version"
    exit 1
fi

OUTFILE="tree-sitter-fixtures-$VERSION.tar.xz"
# WORKDIR="$(mktemp -d tree-sitter-XXXX)"
WORKDIR="${PWD}/tree-sitter-temp-$VERSION"

extract() {
    test -d "$2" || mkdir -p "$2";
    tar xaf "$1" -C "$2" --strip-components=1
}

get () { # args: github_repo, commit, destination
    FILENAME="$(basename "$1")-$2.tar.gz"
    OUT_DIR=${3:-$(basename "$1")-$2}
    echo "Fetching $1@$2 to $OUT_DIR"
    if [ ! -f "$WORKDIR/$FILENAME" ]; then
        curl -LSs  -o "$WORKDIR/$FILENAME" "https://github.com/$1/archive/$2/$FILENAME"
    fi
    rm -rf "$OUT_DIR"
    extract "$WORKDIR/$FILENAME" "$OUT_DIR"
}

# Get date of the commit (tag, sha or any other ref)
get_commit_date () { # args: github_repo, ref
    curl -LSs -H "Accept: application/vnd.github.v3+json" \
            "https://api.github.com/repos/$1/commits/$2" |
        jq -r '.commit.committer.date'
}

# Get last commit in the branch made before the specified timestamp
get_commit_by_date () { # args: github_repo, timestamp, ref (optional)
    curl -LSs -H "Accept: application/vnd.github.v3+json" \
            "https://api.github.com/repos/$1/commits?per_page=1&until=$2${3:+&sha=$3}" |
        jq -r '.[0].sha'
}

# Get test repository commits at the timestamp of the tree-sitter release tag
# (i.e. the exact test data the release tag was tested on)
#
# GitHub API is rate limted, so don't abuse this mode
if [ "$MODE" = "update" ]; then
    TS_TAG_DATE=$(get_commit_date tree-sitter/tree-sitter "v$VERSION")
    echo "# tree-sitter tag date: $TS_TAG_DATE"

    for GRAMMAR in bash c cpp embedded-template go html java \
        javascript jsdoc json php python ruby rust typescript;
    do
        case $GRAMMAR in
            "javascript")
                BRANCH=partial-order-precedences;;
            *)
                BRANCH=master;;
        esac
        COMMIT=$(get_commit_by_date "tree-sitter/tree-sitter-$GRAMMAR" "$TS_TAG_DATE" "$BRANCH")
        printf "grammar %-19s %s\n" "$GRAMMAR" "$COMMIT"
        sleep 1;
    done
    exit 0
fi

mkdir -p "$WORKDIR"
cd "$WORKDIR"

# License: MIT
get tree-sitter/tree-sitter "v$VERSION" "tree-sitter-$VERSION"
cd "tree-sitter-$VERSION"

grammar () { get "tree-sitter/tree-sitter-$1" "${2:-master}" "test/fixtures/grammars/$1"; }
# Verified that all the repositories referenced below are licensed under MIT
# and provide LICENSE files - 2022-01-04
# tree-sitter tag date: 2022-01-01T01:23:08Z
grammar bash                275effdfc0edce774acf7d481f9ea195c6c403cd
grammar c                   e348e8ec5efd3aac020020e4af53d2ff18f393a9
grammar cpp                 e8dcc9d2b404c542fd236ea5f7208f90be8a6e89
grammar embedded-template   d21df11b0ecc6fd211dbe11278e92ef67bd17e97
grammar go                  1203c11e422c73350e672445c5c32b8c0f79266d
grammar html                161a92474a7bb2e9e830e48e76426f38299d99d1
grammar java                ed3a87f750b1d1d533f15ab93fef3e1f5a46e234
grammar javascript          4c2f62903210181ededf461aaa82ca1f804e13e4
grammar jsdoc               189a6a4829beb9cdbe837260653b4a3dfb0cc3db
grammar json                203e239408d642be83edde8988d6e7b20a19f0e8
grammar php                 57f855461aeeca73bd4218754fb26b5ac143f98f
grammar python              24b530ca158d2782ea9046e756057a412e16b52f
grammar ruby                888e2e563ed3b43c417f17e57f7e29c39ce9aeea
grammar rust                d045b04b66d51c0ba8671e7ce1ee23a9f286b7d7
grammar typescript          dfaf3d1468c9c38dd34a02a58a7f8e36252103dd

echo "Writing source archive $DESTDIR/$OUTFILE"
# Drop the examples from the grammar repos. Don't want to check the licenses for unused files
tar caf "$DESTDIR/$OUTFILE" --exclude=examples -v LICENSE test/fixtures
